#ifndef STREAM_H
#define STREAM_H

#include <QPointer>
#include <QThread>

extern "C" {
#include "libavcodec/avcodec.h"
#include "libavformat/avformat.h"
}

#include "VideoSocket.h"

class Stream : public QThread {
    Q_OBJECT
public:
    Stream(QObject *parent = Q_NULLPTR);
    virtual ~Stream();

public:
    static bool init();
    static void deInit();

    void installVideoSocket(VideoSocket *videoSocket);
    bool startDecode();
    void stopDecode();

signals:
    void onStreamStop();
    void getFrame(AVPacket *packet);
    void getConfigFrame(AVPacket *packet);

protected:
    void run();
    bool recvPacket(AVPacket *packet);
    bool pushPacket(AVPacket *packet);
    bool processConfigPacket(AVPacket *packet);
    bool parse(AVPacket *packet);
    bool processFrame(AVPacket *packet);
    qint32 recvData(quint8 *buf, qint32 bufSize);

private:
    QPointer<VideoSocket> m_videoSocket = Q_NULLPTR;

    AVCodecContext *m_codecCtx = Q_NULLPTR;
    AVCodecParserContext *m_parser = Q_NULLPTR;
    // successive packets may need to be concatenated, until a non-config
    // packet is available
    AVPacket* m_pending = Q_NULLPTR;
};

#endif // STREAM_H
